==> xl-react-components-0.16.62.tgz: package/app/xl-react-form/xl-react-dip-dynamic-form-component.js <==
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import moment from 'moment';
import RU from '../utils/ramda-utils';
import {xlMessageTypeProp} from '../xl-props/xl-message-type-prop';
import {XlReactDipBoolean} from '../xl-react-dip/xl-react-dip-boolean/xl-react-dip-boolean';
import {XlReactDipCi} from '../xl-react-dip/xl-react-dip-ci/xl-react-dip-ci';
import {XlReactDipDatetime} from '../xl-react-dip/xl-react-dip-datetime/xl-react-dip-datetime';
import {XlReactDipInteger} from '../xl-react-dip/xl-react-dip-integer/xl-react-dip-integer';
import {XlReactDipEnum} from '../xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';
import {XlReactDipList} from '../xl-react-dip/xl-react-dip-list/xl-react-dip-list';
import {hasInputHints} from '../utils/ci-utils';
import {XlReactDipSetOfCi} from '../xl-react-dip/xl-react-dip-set-of-ci/xl-react-dip-set-of-ci';
import {XlReactDipListOfCi} from '../xl-react-dip/xl-react-dip-list-of-ci/xl-react-dip-list-of-ci';
import {XlReactDipMapStringString} from '../xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string';
import {XlReactDipString} from '../xl-react-dip/xl-react-dip-string/xl-react-dip-string';
import {localeMessages} from '../services/locale-service';
import {XlReactDipSetOfStrings} from '../xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings';
import {XlReactWidgetCategory} from '../xl-react-widget/xl-react-widget-category/xl-react-widget-category';
import {XlReactDipField} from './xl-react-dip-field';
import {getInvalidCategoryNames} from './xl-react-dip-dynamic-form-service';
import {createCiPropertyValidators} from './validator-utils';

export class XlReactDipDynamicFormComponent extends Component {
    static propTypes = {
        allowLookupValues: PropTypes.bool,
        children: PropTypes.oneOfType([
            PropTypes.arrayOf(PropTypes.element),
            PropTypes.element
        ]),
        collapsed: PropTypes.bool,
        dateTimeFormat: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        groupedProperties: PropTypes.array,
        handlers: PropTypes.object,
        message: xlMessageTypeProp,
        metadata: PropTypes.object,
        model: PropTypes.object,
        objLabel: PropTypes.string,
        onLookupToggle: PropTypes.func,
        onMessageChangeHandlerCreator: PropTypes.func,
        onModelChangeCreator: PropTypes.func,
        onPristinityChangeCreator: PropTypes.func,
        onRegister: PropTypes.func,
        onValidityChangeCreator: PropTypes.func,
        pristinity: PropTypes.object,
        toggleCategoryGroup: PropTypes.func,
        validators: PropTypes.arrayOf(PropTypes.func),
        validity: PropTypes.object
    };

    static defaultProps = {allowLookupValues: true};

    state = {};

    createField = (ciProperty) => {
        const standardProperties = this.standardProperties(ciProperty);
        const mkField = (DipField, extraProperties = {}) =>
            <XlReactDipField {...{
                ...standardProperties,
                ...extraProperties,
                fieldComponent: DipField
            }}
            />;
        return R.cond([
            [R.equals('STRING'), () => {
                if (hasInputHints(ciProperty)) {
                    return mkField(XlReactDipEnum);
                }
                return mkField(XlReactDipString, {
                    handlers: this.props.handlers.LOOKUP_CI,
                    onLookupToggle: this.props.onLookupToggle,
                    ...standardProperties
                });
            }],
            [R.equals('INTEGER'), () => mkField(XlReactDipInteger)],
            [R.equals('BOOLEAN'), () => mkField(XlReactDipBoolean)],
            [R.equals('DATE'), () => mkField(XlReactDipDatetime, {
                model: R.is(String, standardProperties.model) ?
                    moment(standardProperties.model, this.props.dateTimeFormat).toDate() :
                    standardProperties.model
            })],
            [R.equals('ENUM'), () => mkField(XlReactDipEnum)],
            [R.equals('SET_OF_STRING'), () => mkField(XlReactDipSetOfStrings)],
            [R.equals('LIST_OF_STRING'), () => mkField(XlReactDipList, {
                handlers: this.props.handlers.LIST_OF_STRING,
                objLabel: this.props.objLabel
            })],
            [R.equals('MAP_STRING_STRING'), () => mkField(XlReactDipMapStringString)],
            [R.equals('SET_OF_CI'), () => mkField(XlReactDipSetOfCi, {
                handlers: this.props.handlers.SET_OF_CI,
                objLabel: this.props.objLabel
            })],
            [R.equals('LIST_OF_CI'), () => mkField(XlReactDipListOfCi, {
                handlers: this.props.handlers.LIST_OF_CI,
                objLabel: this.props.objLabel
            })],
            [R.equals('CI'), () => mkField(XlReactDipCi, {
                handlers: this.props.handlers.CI,
                model: this.getModelOrParent(ciProperty),
                objLabel: this.props.objLabel
            })]
        ])(ciProperty.kind);
    };

    getMetadata = (ciProperty) => {
        const {dateTimeFormat, metadata} = this.props;
        const transformOldProperty = (property) => {
            if (!R.isNil(metadata.required)) {
                return {...property, required: metadata.required};
            }
            return property;
        };
        const transformDefaultDate = (property) => {
            if (R.equals('DATE', property.kind) && !R.isNil(property.default)) {
                return {...property, default: moment(property.default, dateTimeFormat).toDate()};
            }
            return property;
        };
        const transformPasswordProperty = (property) => {
            if (R.equals(property.password, true)) {
                return {...property, type: 'password'};
            }
            return property;
        };
        return R.pipe(
            transformDefaultDate,
            transformOldProperty,
            transformPasswordProperty,
            (property) => ({
                ...property,
                description: `${property.description} (${localeMessages.propertyLabel}: ${property.name})`
            })
        )(ciProperty);
    };

    getModelOrParent = (ciProperty) => ciProperty.asContainment ?
        this.parentCiId(this.props.model) :
        this.props.model[ciProperty.name];

    parentCiId = (ci) => ci.id.substr(0, ci.id.lastIndexOf('/'));

    isDisabled = (ciProperty) => this.props.disabled || ciProperty.asContainment || ciProperty.readonly;

    standardProperties = (ciProperty) => ({
        allowLookupValues: this.props.allowLookupValues,
        disabled: this.isDisabled(ciProperty),
        displayMode: ciProperty.asContainment ? 'visual' : this.props.displayMode,
        message: this.props.message,
        metadata: this.getMetadata(ciProperty),
        model: this.props.model[ciProperty.name],
        onMessageChange: this.props.onMessageChangeHandlerCreator(ciProperty.name),
        onModelChange: this.props.onModelChangeCreator(ciProperty.name),
        onPristinityChange: this.props.onPristinityChangeCreator(ciProperty.name),
        onValidityChange: this.props.onValidityChangeCreator(ciProperty.name),
        register: (value) => this.props.onRegister(ciProperty.name, value),
        validators: createCiPropertyValidators(this.props, ciProperty)
    });

    render() {
        const {children, groupedProperties, toggleCategoryGroup} = this.props;
        const categoryClass = (isInvalid) => classNames({'invalid-category': isInvalid});
        const invalidCategoryNames = getInvalidCategoryNames({
            groupedProperties: this.props.groupedProperties,
            message: this.props.message,
            pristinity: this.props.pristinity,
            validity: this.props.validity
        });

        const categoryPropertiesBlock = (categoryProperties, index) => {
            const categoryClassName = categoryClass(R.includes(categoryProperties.category, invalidCategoryNames));

            const toggleOpen = () => {
                toggleCategoryGroup(categoryProperties.category);
            };

            const displayFormElement = (ciProperty, pInd) => !ciProperty.hidden ?
                <div className="xl-react-dip-dynamic-form-element" key={pInd}>
                    {this.createField(ciProperty)}
                </div> : null;

            return <XlReactWidgetCategory
                categoryClassName={categoryClassName}
                categoryName={categoryProperties.category}
                key={index}
                open={categoryProperties.isOpen}
                toggleOpen={toggleOpen}>
                {RU.mapIndexed(displayFormElement, categoryProperties.properties)}
            </XlReactWidgetCategory>;
        };

        return <div className="xl-react-components xl-react-dip-dynamic-form">
            {RU.mapIndexed(categoryPropertiesBlock, groupedProperties)}
            {children}
        </div>;
    }
}

==> xl-react-components-0.18.103.tgz: package/app/xl-react-form/xl-react-dip-dynamic-form-component.js <==
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import moment from 'moment';
import RU from '../utils/ramda-utils';
import {xlMessageTypeProp} from '../xl-props/xl-message-type-prop';
import {XlReactDipBoolean} from '../xl-react-dip/xl-react-dip-boolean/xl-react-dip-boolean';
import {XlReactDipCi} from '../xl-react-dip/xl-react-dip-ci/xl-react-dip-ci';
import {XlReactDipDatetime} from '../xl-react-dip/xl-react-dip-datetime/xl-react-dip-datetime';
import {XlReactDipInteger} from '../xl-react-dip/xl-react-dip-integer/xl-react-dip-integer';
import {XlReactDipEnum} from '../xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';
import {XlReactDipList} from '../xl-react-dip/xl-react-dip-list/xl-react-dip-list';
import {hasInputHints} from '../utils/ci-utils';
import {XlReactDipSetOfCi} from '../xl-react-dip/xl-react-dip-set-of-ci/xl-react-dip-set-of-ci';
import {XlReactDipListOfCi} from '../xl-react-dip/xl-react-dip-list-of-ci/xl-react-dip-list-of-ci';
import {XlReactDipMapStringString} from '../xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string';
import {XlReactDipString} from '../xl-react-dip/xl-react-dip-string/xl-react-dip-string';
import {localeMessages} from '../services/locale-service';
import {XlReactDipSetOfStrings} from '../xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings';
import {XlReactWidgetCategory} from '../xl-react-widget/xl-react-widget-category/xl-react-widget-category';
import {XlReactDipField} from './xl-react-dip-field';
import {getInvalidCategoryNames} from './xl-react-dip-dynamic-form-service';
import {createCiPropertyValidators} from './validator-utils';

export class XlReactDipDynamicFormComponent extends Component {
    static propTypes = {
        allowLookupValues: PropTypes.bool,
        children: PropTypes.oneOfType([
            PropTypes.arrayOf(PropTypes.element),
            PropTypes.element
        ]),
        collapsed: PropTypes.bool,
        dateTimeFormat: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        groupedProperties: PropTypes.array,
        handlers: PropTypes.object,
        message: xlMessageTypeProp,
        metadata: PropTypes.object,
        model: PropTypes.object,
        objLabel: PropTypes.string,
        onLookupToggle: PropTypes.func,
        onMessageChangeHandlerCreator: PropTypes.func,
        onModelChangeCreator: PropTypes.func,
        onPristinityChangeCreator: PropTypes.func,
        onRegister: PropTypes.func,
        onValidityChangeCreator: PropTypes.func,
        pristinity: PropTypes.object,
        toggleCategoryGroup: PropTypes.func,
        validators: PropTypes.arrayOf(PropTypes.func),
        validity: PropTypes.object
    };

    static defaultProps = {allowLookupValues: true};

    createField = (ciProperty) => {
        const standardProperties = this.standardProperties(ciProperty);
        const mkField = (DipField, extraProperties = {}) =>
            <XlReactDipField {...{
                ...standardProperties,
                ...extraProperties,
                fieldComponent: DipField
            }}
            />;
        return R.cond([
            [R.equals('STRING'), () => {
                if (hasInputHints(ciProperty)) {
                    return mkField(XlReactDipEnum);
                }
                return mkField(XlReactDipString, {
                    handlers: this.props.handlers.LOOKUP_CI,
                    onLookupToggle: this.props.onLookupToggle,
                    ...standardProperties
                });
            }],
            [R.equals('INTEGER'), () => mkField(XlReactDipInteger)],
            [R.equals('BOOLEAN'), () => mkField(XlReactDipBoolean)],
            [R.equals('DATE'), () => mkField(XlReactDipDatetime, {
                model: R.is(String, standardProperties.model) ?
                    moment(standardProperties.model, this.props.dateTimeFormat).toDate() :
                    standardProperties.model
            })],
            [R.equals('ENUM'), () => mkField(XlReactDipEnum)],
            [R.equals('SET_OF_STRING'), () => mkField(XlReactDipSetOfStrings)],
            [R.equals('LIST_OF_STRING'), () => mkField(XlReactDipList, {
                handlers: this.props.handlers.LIST_OF_STRING,
                objLabel: this.props.objLabel
            })],
            [R.equals('MAP_STRING_STRING'), () => mkField(XlReactDipMapStringString)],
            [R.equals('SET_OF_CI'), () => mkField(XlReactDipSetOfCi, {
                handlers: this.props.handlers.SET_OF_CI,
                objLabel: this.props.objLabel
            })],
            [R.equals('LIST_OF_CI'), () => mkField(XlReactDipListOfCi, {
                handlers: this.props.handlers.LIST_OF_CI,
                objLabel: this.props.objLabel
            })],
            [R.equals('CI'), () => mkField(XlReactDipCi, {
                handlers: this.props.handlers.CI,
                model: this.getModelOrParent(ciProperty),
                objLabel: this.props.objLabel
            })]
        ])(ciProperty.kind);
    };

    getMetadata = (ciProperty) => {
        const {dateTimeFormat, metadata} = this.props;
        const transformOldProperty = (property) => {
            if (!R.isNil(metadata.required)) {
                return {...property, required: metadata.required};
            }
            return property;
        };
        const transformDefaultDate = (property) => {
            if (R.equals('DATE', property.kind) && !R.isNil(property.default)) {
                return {...property, default: moment(property.default, dateTimeFormat).toDate()};
            }
            return property;
        };
        const transformPasswordProperty = (property) => {
            if (R.equals(property.password, true)) {
                return {...property, type: 'password'};
            }
            return property;
        };
        return R.pipe(
            transformDefaultDate,
            transformOldProperty,
            transformPasswordProperty,
            (property) => ({
                ...property,
                description: `${property.description} (${localeMessages.propertyLabel}: ${property.name})`
            })
        )(ciProperty);
    };

    getModelOrParent = (ciProperty) => ciProperty.asContainment ?
        this.parentCiId(this.props.model) :
        this.props.model[ciProperty.name];

    parentCiId = (ci) => ci.id.substr(0, ci.id.lastIndexOf('/'));

    isDisabled = (ciProperty) => this.props.disabled || ciProperty.asContainment || ciProperty.readonly;

    standardProperties = (ciProperty) => ({
        allowLookupValues: this.props.allowLookupValues,
        disabled: this.isDisabled(ciProperty),
        displayMode: ciProperty.asContainment ? 'visual' : this.props.displayMode,
        message: this.props.message,
        metadata: this.getMetadata(ciProperty),
        model: this.props.model[ciProperty.name],
        onMessageChange: this.props.onMessageChangeHandlerCreator(ciProperty.name),
        onModelChange: this.props.onModelChangeCreator(ciProperty.name),
        onPristinityChange: this.props.onPristinityChangeCreator(ciProperty.name),
        onValidityChange: this.props.onValidityChangeCreator(ciProperty.name),
        register: (value) => this.props.onRegister(ciProperty.name, value),
        validators: createCiPropertyValidators(this.props, ciProperty)
    });

    render() {
        const {children, groupedProperties, toggleCategoryGroup} = this.props;
        const categoryClass = (isInvalid) => classNames({'invalid-category': isInvalid});
        const invalidCategoryNames = getInvalidCategoryNames({
            groupedProperties: this.props.groupedProperties,
            message: this.props.message,
            pristinity: this.props.pristinity,
            validity: this.props.validity
        });

        const categoryPropertiesBlock = (categoryProperties, index) => {
            const categoryClassName = categoryClass(R.includes(categoryProperties.category, invalidCategoryNames));

            const toggleOpen = () => {
                toggleCategoryGroup(categoryProperties.category);
            };

            const displayFormElement = (ciProperty, pInd) => !ciProperty.hidden ?
                <div className="xl-react-dip-dynamic-form-element" key={pInd}>
                    {this.createField(ciProperty)}
                </div> : null;

            return <XlReactWidgetCategory
                categoryClassName={categoryClassName}
                categoryName={categoryProperties.category}
                key={index}
                open={categoryProperties.isOpen}
                toggleOpen={toggleOpen}
                {...categoryProperties.categoryOptions}>
                {RU.mapIndexed(displayFormElement, categoryProperties.properties)}
            </XlReactWidgetCategory>;
        };

        return <div className="xl-react-components xl-react-dip-dynamic-form">
            {RU.mapIndexed(categoryPropertiesBlock, groupedProperties)}
            {children}
        </div>;
    }
}
